<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\AnaliseCotacoes\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class Sup03400ConsultaDeMelhoresPrecosController extends \Zeedhi\Framework\Controller\Report {


    protected $entityManager;
    protected $qr2Strategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $Sup03400ConsultaDeMelhoresPrecosService;
    protected $reportMapping;
    protected $birtStrategy;

    public function __construct($reportService, $entityManager, $qr2Strategy, $newCode, $connection, $environment, $Sup03400ConsultaDeMelhoresPrecosService,$birtStrategy) {
        $this->reportService  = $reportService;
        $this->entityManager  = $entityManager;
        $this->qr2Strategy    = $qr2Strategy;
        $this->newCode        = $newCode;
        $this->connection     = $connection;
        $this->environment    = $environment;
        $this->Sup03400ConsultaDeMelhoresPrecosService = $Sup03400ConsultaDeMelhoresPrecosService;
        $this->birtStrategy = $birtStrategy;
    }
    
    public function findSubgrupo(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $cdfilial = $this->environment->getCdFilial();
            $dataSet = $this->Sup03400ConsultaDeMelhoresPrecosService->getSubgrupos($cdfilial);
            $response->addDataSet(new DataSet('subgrupprod_filial', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $cdfilial = $this->environment->getCdFilial();
        $row["P_CDFILIAL"] = $this->Sup03400ConsultaDeMelhoresPrecosService->getCdFilial($cdfilial);
        //fornecedor
        if($row["L_CDFORNECED"]=='T'){
             $row['L_CDFORNECED']=$this->retornaFornecedores();
        }
        $row['L_CDFORNECED']=$this->criarListaValores($row['L_CDFORNECED']);
        //operador
        if($row["L_CDOPERADOR"]=='T'){
             $row['L_CDOPERADOR']=$this->retornaCompradores();
        }
        $row['L_CDOPERADOR']=$this->criarListaValores($row['L_CDOPERADOR']);
       //regiao
        if($row["L_CDREGIAO"]=='T'){
             $row['L_CDREGIAO']=$this->retornaRegiao();
        }
        $row['L_CDREGIAO']=$this->criarListaValores($row['L_CDREGIAO']);
        $row['L_CDREGIAO1']= $row['L_CDREGIAO'];
        $row['L_CDREGIAO2']= $row['L_CDREGIAO'];
        $row['L_CDREGIAO3']= $row['L_CDREGIAO'];
        $row['L_CDREGIAO4']= $row['L_CDREGIAO'];
        $row['__report_name'] = $row['REPORT_NAME'];
        $row['P_NRORG'] = $this->environment->getNrorgTrab();
        $this->initializeReportMapping();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        
    }
    
    public function insertLista($nrlista,$cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
      //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
       
            $this->entityManager->beginTransaction();
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                $this->entityManager->commit();
                return $nrlista;
            } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        
    }
    
  
    
    public function initializeReportMapping() {
        $params = array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
                array (
                    "L_CDFORNECED"   => "L_CDFORNECED",
                    "L_CDOPERADOR"   => "L_CDOPERADOR",
                    "L_CDREGIAO"     => "L_CDREGIAO",
                    "L_CDREGIAO1"    => "L_CDREGIAO1",
                    "L_CDREGIAO2"    => "L_CDREGIAO2",
                    "L_CDREGIAO3"    => "L_CDREGIAO3",
                    "L_CDREGIAO4"    => "L_CDREGIAO4",
                    "P_CDFILIAL"     => "P_CDFILIAL",
                    "P_CDPRODFIN"    => "P_CDPRODFIN",
                    "P_CDPRODFINAL"  => "P_CDPRODFINAL",
                    "P_CDPRODINI"    => "P_CDPRODINI",
                    "P_DTFIN"        => "P_DTFIN",
                    "P_DTINI"        => "P_DTINI",
                    "P_SUBGRUPO"     => "P_SUBGRUPO",
                    "P_TCDPRODUTO"   => "P_TCDPRODUTO",
                    "P_TPROD"        => "P_TPROD",
                    "P_VALIDAS"      => "P_VALIDAS",
                    "P_VENCIDAS"     => "P_VENCIDAS",
                    "P_CDPICTPROD"   => "P_CDPICTPROD",
                    "P_NRORG"        => "P_NRORG"
                ),
        );
        
        $this->reportMapping = array (
            'SUP03400' => $params,
            'SUP03401' => $params,
            'SUP03402' => $params,
            'SUP03403' => $params,
            'SUP03404' => $params,
            'SUP03405' => $params,
            'SUP03406' => $params,
            'SUP03407' => $params,
            'SUP03408' => $params,
            'SUP03409' => $params,
            'SUP0340A' => $params,
            'SUP0340B' => $params,
            'SUP0340C' => $params,
            'SUP0340D' => $params,
            'SUP0340F' => $params,
            'SUP0340G' => $params,
            'SUP0340H' => $params
        );
    }
    
      //retorna array de operadores
    public function retornaCompradores(){
         $sql = "SELECT COMPRADOR.CDOPERADOR FROM COMPRADOR, OPERADOR WHERE(COMPRADOR.CDOPERADOR = OPERADOR.CDOPERADOR) ORDER BY COMPRADOR.CDOPERADOR";
        try {
            $comprador = $this->entityManager->getConnection()->fetchAll($sql);
            return implode('_', array_column($comprador, 'CDOPERADOR'));
        } catch (\Exception $e) {
            throw $e;
        }
     }
        //retorna array de fornecedores
    public function retornaFornecedores(){
         $sql = "SELECT CDFORNECED FROM FORNECEDOR";

        try {
            $fornecedor = $this->entityManager->getConnection()->fetchAll($sql);
            return implode('_', array_column($fornecedor, 'CDFORNECED'));
        } catch (\Exception $e) {
            throw $e;
        }
     }
     	//retorna array de regiões
     public function retornaRegiao(){
         $sql = "SELECT CDREGIAO FROM REGIAO";

        try {
            $regiao = $this->entityManager->getConnection()->fetchAll($sql);
            return implode('_', array_column($regiao, 'CDREGIAO'));
        } catch (\Exception $e) {
            throw $e;
        }
     }
}